/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.example.demo;

import lombok.extern.slf4j.Slf4j;
import org.springframework.context.event.ContextRefreshedEvent;
import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Component;
import reactor.core.publisher.Flux;

/**
 * @author hantsy
 */
@Component
@Slf4j
class DataInitializer {

    private final PostRepository posts;

    public DataInitializer(PostRepository posts) {
        this.posts = posts;
    }

    @EventListener(value = ContextRefreshedEvent.class)
    public void init() {
        log.info("start data initialization  ...");
        this.posts
                .deleteAll()
                .thenMany(
                        Flux
                                .range(0, 100)
                                .flatMap(
                                        n -> this.posts.save(
                                                Post.builder()
                                                        .title("post title #" + n)
                                                        .content("content of post " + n)
                                                        .build()
                                        )
                                )
                )
                .log()
                .subscribe(
                        data -> log.info("data: {}", data),
                        error -> log.info("error: {}", error),
                        () -> log.info("done initialization...")
                );

    }

}